import{_ as p}from"./DbInitComponent.vue_vue_type_script_setup_true_lang-qRj5xf5o.js";import{d as f,r as x,b as g,i as e,w as t,h as n,o as i,c as y,k as v,e as o,f as _}from"./index-Ua38_nuB.js";const b={class:"common-layout",style:{height:"100vh",width:"100%","background-color":"#F7F7F7"}},w=o("h2",{style:{"text-align":"center"}},[_("欢迎使用"),o("a",{href:"https://github.com/xaoyaoo/PyWxDump.git"},"PyWxDump"),_("聊天记录查看工具! "),o("span",{style:{"font-size":"14px"}},"(如需提前体验更多功能请开通超级vip)")],-1),k=o("span",{style:{color:"#043bea"}},"设置",-1),F=o("span",null,"初始化设置",-1),V=o("span",null,null,-1),D=f({__name:"SettingView",setup(B){const l=x(""),d=s=>{l.value=s};return(s,C)=>{const h=n("el-header"),a=n("el-menu-item"),m=n("el-menu"),r=n("el-aside"),u=n("el-main"),c=n("el-container");return i(),g("div",b,[e(c,null,{default:t(()=>[e(h,{style:{height:"35px","max-height":"35px",width:"100%"}},{default:t(()=>[w]),_:1}),e(c,{style:{height:"calc(100vh - 35px)",width:"100%"}},{default:t(()=>[e(r,{width:"200px",style:{height:"100%"}},{default:t(()=>[e(m,{style:{height:"100%","background-color":"#F7F7F7",color:"#262626"},"default-active":"2",class:"el-menu-vertical-demo",onSelect:d},{default:t(()=>[e(a,{index:"-1",disabled:""},{default:t(()=>[k]),_:1}),e(a,{index:"db_init"},{default:t(()=>[F]),_:1}),e(a,{index:"3"},{default:t(()=>[V]),_:1})]),_:1})]),_:1}),e(u,{style:{height:"100%","max-height":"100%",width:"100%",margin:"0",padding:"0"}},{default:t(()=>[l.value=="db_init"?(i(),y(p,{key:0})):v("",!0)]),_:1})]),_:1})]),_:1})])}}});export{D as default};
